struct stretchyBones
(
	fn stretchyBonesFN=
	(
		/*This script was designed to improve workflow when dealing with bones.
		It's important that you double-click on the chain's root bone to get your selection.
		Script made by Jason Labb
		Credit goes to Paul Neale
		This script and more at www.jasonlabbe3d.com
		If you are using this tool, I'd appreciate any feedback, which can be fowarded to jasondlabbe@hotmail.com
		*/

		rollout rolloutDisplay "Jason Labbe Stretchy Bones Tool" rolledUp:false
		(
			--Temporary variables
			local ptHelperArray = #()
			local ptHelper = point
			local helperArrayIndex = 1
			local rollState = 0
			local transformVal = #()
			local transformValIndex = 1
			local ConArrayIndex = 1
			local selectionArray = #()
			local ConArray = #()
			local parentVar = ()
			local linkPts = false
	
			--Helper Display Interface
			group "Display Options"
			(
				spinner ptHelperSize range:[0,100,25] fieldWidth:35 pos:[10,25] label labelPtSize "Size" pos:[62,25]
				checkbox displayCenterMarker "Center Marker"
				checkbox displayAxisTripod "Axis Tripod"
				checkbox displayCross "Cross"
				checkbox displayBox "Box" checked:true
				colorPicker ptHelperColor "Color" color:[255,0,0] fieldWidth:20
				radioButtons stretchVal "Bone Stretch Value" labels:# ("Scale", "Squash")
				checkbox linkPtsChk ":Link Points"
			)	

			--Button Interface
			group "Alignment Options"	
			(
				button alignPosOriBut "Create Stretchy Bones" width:120
				button undoBut "Undo Last Action" width:120
			)

			--FUNCTION: Get temporary transforms
			fn getTransformsFN =
			(
				transformVal = #()
				selectionArray = #()
				conArray = #()
				parentVar = ()
				for i in selection do
				(
					append transformVal i.transform
					append conArray i.position.controller
					append conArray i.rotation.controller
					append selectionArray i
				)
				parentVar = $selection[1].parent
			)

			--FUNCTION: Point Helper Display Properties
			fn helperDisplayFN =
			(
				ptHelper.size = ptHelperSize.value
				ptHelper.centerMarker = displayCenterMarker.state
				ptHelper.axisTripod = displayAxisTripod.state
				ptHelper.cross = displayCross.state
				ptHelper.box = displayBox.state
				ptHelper.wireColor = ptHelperColor.color	
			)

			--FUNCTION: Producing and aligning in position and orientation
			fn alignPosOriFN =
			(
				ptHelperArray = #()
				for i in selection do
				(
					i.position.controller = position_list()
					i.position.controller.available.controller = position_xyz()
					i.position.controller.setName 1 "Static Position"
					i.position.controller.setName 2 "Zero Position"
					i.rotation.controller = rotation_list()
					i.rotation.controller.available.controller = euler_xyz()
					i.rotation.controller.setName 1 "Static Rotation"
					i.rotation.controller.setName 2 "Zero Rotation"
				
					if stretchVal.state == 1 then
					i.boneScaleType = #scale else
					i.boneScaleType = #squash
							
					ptHelper = point()
					helperDisplayFN()
					ptHelper.rotation = i.rotation
					ptHelper.pos = i.pos
					ptHelper.position.controller = position_list()
					ptHelper.position.controller.available.controller = position_xyz()
					ptHelper.position.controller.setName 1 "Static Position"
					ptHelper.position.controller.setName 2 "Zero Position"
					ptHelper.rotation.controller = rotation_list()
					ptHelper.rotation.controller.available.controller = euler_xyz()
					ptHelper.rotation.controller.setName 1 "Static Rotation"
					ptHelper.rotation.controller.setName 2 "Zero Rotation"
					ptHelper.position.controller.setActive 2
					ptHelper.rotation.controller.setActive 2
					i.position.controller.available.controller = position_constraint()
					i.position.controller.setName 3 "Constraint: Position"
					i.position.controller["Constraint__Position"].appendTarget ptHelper 100
					i.position.controller.setActive 3
					append ptHelperArray ptHelper
				)	
						
				ptHelperArray[1].parent = parentVar
						
				-- Will re-create the bone heirarchy if the checkbox is checked.
				if linkPts then for i = 2 to ptHelperArray.count do
				(
					ptHelperArray[i].parent = ptHelperArray[i-1]
				)
						
				for i in selection do
				(	
					if helperArrayIndex+1<= ptHelperArray.count then
					(	
						i.rotation.controller.available.controller = lookAt_constraint()
						i.rotation.controller.setName 3 "Constraint: LookAt"	
						i.rotation.controller["Constraint__LookAt"].appendTarget ptHelperArray[(helperArrayIndex+1)] 100
						i.rotation.controller["Constraint__LookAt"].lookAt_vector_length=0
						i.rotation.controller["Constraint__LookAt"].upNode_world = false
						i.rotation.controller.setActive 3
					)
					if helperArrayIndex!= ptHelperArray.count then	
					i.rotation.controller["Constraint__LookAt"].pickUpNode = ptHelperArray[helperArrayIndex]			
					helperArrayIndex = helperArrayIndex+1
				)			
			)

			--FUNCTION: Undo
			fn undoFN =
			(
				transformValIndex = 1
				conArrayIndex = 1
				for i in selectionArray do
				(
					i.position.controller = conArray[conArrayIndex]
					i.rotation.controller = conArray[conArrayIndex+1]
					i.transform = transformVal[0+transformValIndex]
					transformValIndex = transformValIndex+1
					conArrayIndex = ConArrayIndex + 2
				)
				try(delete ptHelperArray)catch()
			)

			--BUTTON: Align helpers
			on alignPosOriBut pressed do
			(
				if $ != undefined then
				(
					if $selection[1].position.controller["static_position"] == undefined and $selection[1].rotation.controller["static_rotation"] == undefined then
					(
						print "Valid selection"
						helperArrayIndex = 1
						getTransformsFN()
						alignPosOriFN()
					)
					else print "Invalid selection: Stretchy Bone(s) already made."
				)
				else print "Invalid Selection: No selection"
			)
	
			--CHECKBOX: Links the points.
			on linkPtsChk changed val do
			(
				linkPts = val
			)
						
			--BUTTON: Undo
			on undoBut pressed do 
			(
				undoFN()
			)	

			on rolloutDisplay rolledUp rollState do 
				(
					if rollState != true then 
					mainRollout.size = [175, 50]
					else mainRollout.size = [175, 320]
				)
		)
	),
	
	--Script Info
	fn info=
	(
		rollout rolloutInfo "About Stretchy Bones Tool" rolledUp:true
		(
			groupBox groupTool "About the tool" pos:[4,5] width:350 height:90
			label labelTool1 "This tool allows the user to collect a selection of bones, then make them into 'stretchy bones' within a click of a button. BUGS: You have to double click on the first bone to get your selection. Selecting the bones in random order will mess up the selection array and give strange results." pos:[7,26] width:340 height:90
			group "About the Author"
			(
				label labelInfo1 "Made by Jason Labb"
				label labelInfo2 "Credit to Paul Neale"
				label labelInfo3 "www.jasonlabbe3d.com"
				label labelInfo4 "jasondlabbe@hotmail.com"
			)
		)
	),

	fn getRollouts=#(stretchyBonesFN(), info())--getRollouts needs to be ther for every tool we write for path. OTHERWISE WONT WOR
)
	stretchyBones=stretchyBones()
